"""
Embedded Python Blocks:

Each time this file is saved, GRC will instantiate the first class it finds
to get ports and parameters of your block. The arguments to __init__  will
be the parameters. All of them are required to have default values!
"""

from collections import namedtuple
import numpy as np
from gnuradio import gr
import pmt

from goessdr.protocol import ProtocolProcessor as pp


class ToProtoProcessor(gr.sync_block):  # other base classes are basic_block, decim_block, interp_block
    """Embedded Python Block example - a simple multiply const"""

    def __init__(self, product_dir = "", xrit_imagery_filter=True, emwin_filter=True, dcs_filter=True, frame_size=100, debug=False):  # only default arguments here
        """arguments to this function show up as parameters in GRC"""
        gr.sync_block.__init__(
            self,
            name='Protocol Processor',   # will show up in GRC
            in_sig=[np.uint8],
            out_sig=[]
        )
        # if an attribute with the same name as a parameter is found,
        # a callback is registered (properties work, too).
        #self.example_param = example_param
        self.product_dir = product_dir
        self.xrit_imagery_filter = xrit_imagery_filter
        self.emwin_filter = emwin_filter
        self.dcs_filter = dcs_filter
        self.frame_size = frame_size
        self.set_output_multiple(frame_size)
        self.debug = debug
        self.message_port_register_in(pmt.intern("report"))
        self.message_port_register_in(pmt.intern("reset"))
        self.message_port_register_out(pmt.intern("report"))
        self.set_msg_handler(pmt.intern("report"), self.report_error_counts) # No bind necessary, we can pass the function directly
        self.set_msg_handler(pmt.intern("reset"), self.reset_error_counts)

    def forecast(self, noutput_items, ninput_items_required):
        for i, _ in enumerate(ninput_items_required):
            ninput_items_required[i] = noutput_items

    def report_error_counts(self,msg_pmt):
        if pmt.is_true(msg_pmt):
            missed_vcdu, tot_vcdu = pp.report_error_counts()
            tup = pmt.make_tuple(pmt.to_pmt(missed_vcdu),pmt.to_pmt(tot_vcdu))
            msg = pmt.make_dict()
            msg = pmt.dict_add(msg, pmt.intern("goes errors"), tup)
            self.message_port_pub(pmt.intern("report"), msg)

    def reset_error_counts(self,msg_pmt):
        if pmt.is_true(msg_pmt):
            pp.reset_error_counts()

    def fixed_rate_noutput_to_ninput(self, noutput):
        return noutput

    def print_debug(self, ninput_items, tags):
        print "Input Frame: %d bytes," % ninput_items,
        if len(tags):
            offset0, tag0 = min(tags.iteritems(), key=lambda x: x[0])
            print "%d tags -- tag[0] @ %d: (%s, %d)" % (
                len(tags), offset0, tag0.key, tag0.value)
        else:
            print "no tags"


    TagTuple = namedtuple('TagTuple', 'key value')
    def work(self, input_items, output_items):
        """send frames to proto processor"""
        ninput_items = input_items[0].size
        tags = self.get_tags_in_range(0, self.nitems_read(0),
                                      self.nitems_read(0) + ninput_items)
        ttags = {tag.offset - self.nitems_read(0):
                    self.TagTuple(pmt.to_python(tag.key), pmt.to_python(tag.value))
                 for tag in tags}
        
        self.product_dict = {'0':self.xrit_imagery_filter,'2':self.emwin_filter,'129':self.dcs_filter,'214':self.emwin_filter,'130':self.dcs_filter}
        if self.debug:
            self.print_debug(ninput_items, ttags)
        pp.protoproc(input_items[0], ttags, self.product_dir, self.product_dict)

        return ninput_items
