#from multiprocessing import Process
from threading import Thread
from CADULayer import CADULayer
from VCDULayer import VCDULayer
from MPDULayer import MPDULayer
from CPPDULayerPort import CPPDULayerPort
from PPDULayer import PPDULayer
from EMWINLayer import EMWINLayer
from PrintLayer import PrintLayer

class LritReceiver(object):

   CADU_CODEWORD_LENGTH = 1024
   #VCDU_CODEWORD_LENGTH = 1020

   def __init__(self):
      self.derandom = CADULayer()
      self.vcdu = VCDULayer()
      self.mpdu = MPDULayer()
      self.cppdu = CPPDULayerPort()
      self.ppdu = PPDULayer()
      self.emwinLayer = EMWINLayer()
      self.printLayer = PrintLayer()

      self.derandom.connect(self.vcdu.getInBuffer())
      self.vcdu.connect(self.mpdu.getInBuffer())
      self.mpdu.connect(self.cppdu.getInBuffer())
      self.cppdu.connect(self.ppdu.getInBuffer())
      self.ppdu.connect(self.printLayer.getInBuffer())
      #EMWIN is an optional layer
      self.ppdu.connect(self.emwinLayer.getInBuffer())
      self.emwinLayer.connect(self.printLayer.getInBuffer())

      self.pDerandom = Thread(target=self.derandom.threadFunction)
      self.pVcdu = Thread(target=self.vcdu.threadFunction)
      self.pMpdu = Thread(target=self.mpdu.threadFunction)
      self.pCppdu = Thread(target=self.cppdu.threadFunction)
      self.pPpdu = Thread(target=self.ppdu.threadFunction)
      self.pPrintLayer = Thread(target=self.printLayer.threadFunction)
      self.pEmwinLayer = Thread(target=self.emwinLayer.threadFunction)

      self.pDerandom.start()
      self.pVcdu.start()
      self.pMpdu.start()
      self.pCppdu.start()
      self.pPpdu.start()
      self.pPrintLayer.start()
      self.pEmwinLayer.start()

   def __del__(self):
      self.derandom.getInBuffer().put((None,None))
      #self.vcdu.getInBuffer().put((None,None))

      self.pDerandom.join()
      self.pVcdu.join()
      self.pMpdu.join()
      self.pCppdu.join()
      self.pPpdu.join()
      self.pEmwinLayer.join()
      self.pPrintLayer.join()

   def rx(self,data):
      if len(data) == self.CADU_CODEWORD_LENGTH:
         self.derandom.getInBuffer().put((data,None))
      else:
         raise ValueError("Wrong data size: Expected {0}, got {1}".format(self.CADU_CODEWORD_LENGTH,len(data)))


