from GOESFrame import GOESFrame

##
# @brief MPDU Message class.
# @details Contains a single MP Data Unit. The only interesting header
# information is the firstHeaderPointer, which is a pointer to the first
# byte in the MPDU that contains a new Transport Frame.  This is
# important because the CPPDU layer depends on this value being useful and
# correct!
# @author Miguel Dominguez
# @date 2016
class MPDUMsg(GOESFrame):
    SPARE_MASK = 0xF8
    SPARE_SHIFT = 3
    FIRST_HEADER_POINTER_MASK_LEFT = 0x07
    FIRST_HEADER_POINTER_SHIFT_LEFT = 8
    FIRST_HEADER_POINTER_MASK_RIGHT = 0xFF

    HEADER_SIZE_BYTES = 2
    DATA_SIZE_BYTES = 884

    NOT_NEW_FRAME = 2047

    ##
    # @fn getHeader
    # @brief Extracts the header values with appropriate bit masking and shifting
    # @param[in] headerBytes Unsigned byte list sufficiently long enough to
    # contain the header (2 bytes)
    def getHeader(self, headerBytes):
        self.spare = (headerBytes[0] & self.SPARE_MASK) >> self.SPARE_SHIFT
        self.firstHeaderPointer = ((headerBytes[0] & self.FIRST_HEADER_POINTER_MASK_LEFT) << self.FIRST_HEADER_POINTER_SHIFT_LEFT)
        self.firstHeaderPointer |= (headerBytes[1] & self.FIRST_HEADER_POINTER_MASK_RIGHT)

    ##
    # @fn fromFrame
    # @brief Extracts the MPDU Message from the VCDUMsg
    # @details This also calls getHeader
    # @param[in] data from a VCDUMsg
    def fromFrame(self, data):
        self.getHeader(data[0:self.HEADER_SIZE_BYTES])
        self.data = data[self.HEADER_SIZE_BYTES:len(data)]

    ##
    # @fn __str__
    # @brief Returns the First Header Pointer as a string
    # @param[out] str Header information as a string
    def __str__(self):
        output = "First Header Pointer: {0}".format(self.firstHeaderPointer)
        #output += "".join([chr(x) for x in self.data])
        return output
